@@ -26,6 +26,8 @@
 #include <sys/socket.h>
 #include <sys/syslog.h>
 #include <sys/protosw.h>
+#include <kern/locks.h>
+
 #include <net/if_types.h>
 #include <net/route.h>
 #include <net/dlil.h>
@@ -48,9 +50,8 @@ Definitions
 /* -----------------------------------------------------------------------------
 Declarations
 ----------------------------------------------------------------------------- */
-extern void    m_copydata __P((struct mbuf *, int, int, caddr_t));
 
-void	pptp_ip_input(struct mbuf *, int len);
+void	pptp_ip_input(mbuf_t , int len);
 
 /* -----------------------------------------------------------------------------
 Globals
@@ -58,6 +59,10 @@ Globals
 
 struct protosw 	gre_pr, *old_pr;
 
+extern lck_mtx_t	*ip_mutex;
+extern lck_mtx_t	*ppp_domain_mutex;
+extern lck_mtx_t	*inet_domain_mutex;
+
 /* -----------------------------------------------------------------------------
 intialize pptp datalink attachment strutures
 ----------------------------------------------------------------------------- */
@@ -65,9 +70,14 @@ int pptp_ip_init()
 {
 
     bzero(&gre_pr, sizeof(gre_pr));
-    gre_pr.pr_input = pptp_ip_input;
+    gre_pr.pr_input = (void	(*)(struct mbuf *, int))pptp_ip_input;
+    gre_pr.pr_flags = PR_PROTOLOCK;
+	lck_mtx_unlock(ppp_domain_mutex);
+	lck_mtx_lock(ip_mutex);
     old_pr = ip_protox[IPPROTO_GRE];
     ip_protox[IPPROTO_GRE] = &gre_pr;
+	lck_mtx_unlock(ip_mutex);
+	lck_mtx_lock(ppp_domain_mutex);
     return 0;
 }
 
@@ -77,24 +87,28 @@ can't dispose if clients are still attached
 ----------------------------------------------------------------------------- */
 int pptp_ip_dispose()
 {
+	lck_mtx_unlock(ppp_domain_mutex);
+	lck_mtx_lock(ip_mutex);
     ip_protox[IPPROTO_GRE] = old_pr;
+	lck_mtx_unlock(ip_mutex);
+	lck_mtx_lock(ppp_domain_mutex);
     return 0;
 }
 
 /* -----------------------------------------------------------------------------
 callback from ip
 ----------------------------------------------------------------------------- */
-void pptp_ip_input(struct mbuf *m, int len)
+void pptp_ip_input(mbuf_t m, int len)
 {
     struct ip 		*ip;
     u_int32_t 		from;
-    struct mbuf 	*m1;
+    mbuf_t			m1;
 
 #if 0
     u_int8_t 		*d, i;
     d = mtod(m, u_int8_t *);
     for (i = 0; i < 64; i+=16) {
-    log(LOG_INFO, "pptp_ip_input: data 0x %x %x %x %x %x %x %x %x - %x %x %x %x %x %x %x %x\n",
+    log(LOGVAL, "pptp_ip_input: data 0x %x %x %x %x %x %x %x %x - %x %x %x %x %x %x %x %x\n",
         d[i+0],d[i+1],d[i+2],d[i+3],d[i+4],d[i+5], d[i+6], d[i+7],
         d[i+8], d[i+9], d[i+10], d[i+11], d[i+12], d[i+13], d[i+14], d[i+15]);
     }
@@ -106,36 +120,44 @@ void pptp_ip_input(struct mbuf *m, int len)
     to mbuf chain, neither data compression nor IP layer
     IP layer seems to expext IP header in a contiguous block
     */
-    m1 = m_getpacket();
-    if (m1 == 0)
+   if (mbuf_getpacket(MBUF_WAITOK, &m1) != 0)
         goto fail;    
-    m_copydata(m, 0, m->m_pkthdr.len, mtod(m1, caddr_t));
-    m1->m_len = m->m_pkthdr.len;
-    m1->m_pkthdr.len = m->m_pkthdr.len;
+    mbuf_copydata(m, 0, mbuf_pkthdr_len(m), mbuf_data(m1));
+    mbuf_setlen(m1, mbuf_pkthdr_len(m));
+    mbuf_pkthdr_setlen(m1, mbuf_pkthdr_len(m));
     
-    ip = mtod(m1, struct ip *);
+    ip = mbuf_data(m1);
     from = ip->ip_src.s_addr;
 
     /* remove the IP header */
-    m_adj(m1, ip->ip_hl * 4);
+    mbuf_adj(m1, ip->ip_hl * 4);
 
+	lck_mtx_lock(ppp_domain_mutex);
     if (pptp_rfc_lower_input(m1, from)) {
+		lck_mtx_unlock(ppp_domain_mutex);
         // success, free the original mbuf
-        m_freem(m);
+        mbuf_freem(m);
         return;
     }
-
+	lck_mtx_unlock(ppp_domain_mutex);
+	
 fail:
     // the packet was not for us, just call the old hook
     if (m1)
-	m_freem(m1);
-    (*old_pr).pr_input(m, len);
+	mbuf_freem(m1);
+	
+	if (!((*old_pr).pr_flags & PR_PROTOLOCK)) {
+		lck_mtx_lock(inet_domain_mutex);
+		(*old_pr).pr_input((struct mbuf *)m, len);
+		lck_mtx_unlock(inet_domain_mutex);
+	} else
+		(*old_pr).pr_input((struct mbuf *)m, len);
 }
 
 /* -----------------------------------------------------------------------------
 called from pppenet_proto when data need to be sent
 ----------------------------------------------------------------------------- */
-int pptp_ip_output(struct mbuf *m, u_int32_t from, u_int32_t to)
+int pptp_ip_output(mbuf_t m, u_int32_t from, u_int32_t to)
 {
     struct route ro;
     struct ip 	*ip;
@@ -145,27 +167,28 @@ int pptp_ip_output(struct mbuf *m, u_int32_t from, u_int32_t to)
 
     d = mtod(m, u_int8_t *);
     for (i = 0; i < 64; i+=16) {
-    log(LOG_INFO, "pptp_ip_output: data 0x %x %x %x %x %x %x %x %x - %x %x %x %x %x %x %x %x\n",
+    log(LOGVAL, "pptp_ip_output: data 0x %x %x %x %x %x %x %x %x - %x %x %x %x %x %x %x %x\n",
         d[i+0],d[i+1],d[i+2],d[i+3],d[i+4],d[i+5], d[i+6], d[i+7],
         d[i+8], d[i+9], d[i+10], d[i+11], d[i+12], d[i+13], d[i+14], d[i+15]);
     }
 #endif
 
-    M_PREPEND(m, sizeof(struct ip), M_WAIT);
-    if (m == 0)
+    if (mbuf_prepend(&m, sizeof(struct ip), MBUF_WAITOK) != 0)
         return 1;
         
-    ip = mtod(m, struct ip *);
+    ip = mbuf_data(m);
     ip->ip_tos = 0;
     ip->ip_off = 0;
     ip->ip_p = IPPROTO_GRE;
-    ip->ip_len = m->m_pkthdr.len;
+    ip->ip_len = mbuf_pkthdr_len(m);
     ip->ip_src.s_addr = from;
     ip->ip_dst.s_addr = to;
     ip->ip_ttl = MAXTTL;
  
     bzero(&ro, sizeof(ro));
-    ip_output(m, 0, &ro, 0, 0);
+	lck_mtx_unlock(ppp_domain_mutex);
+    ip_output((struct mbuf *)m, 0, &ro, 0, 0);
+	lck_mtx_lock(ppp_domain_mutex);
     return 0;
 }
 
